"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const child_process_1 = require("child_process");
const gitLogArgs = (range) => 
// tslint:disable-next-line max-line-length
['log', range, '--format=\x1F%H\x1F%h\x1F%an\x1F%ae\x1F%at\x1F%cn\x1F%ce\x1F%ct\x1F%s\x1F%b\x1F\x1E'];
/**
 *  Asynchronously fetches the metadata of all commits within a particular reference range.
 *  @param dir The path to the root directory of a git repository.
 *  @param startRef A reference string, such as a commit hash or tag name, which designates the
 *    beginning of the range (exclusive.) If not defined, all ancestors of endRef will be listed.
 *  @param endRef A reference string, such as a commit hash or tag name, which designates the end
 *    of the range (inclusive.) If not defined, endRef will be assumed to be 'HEAD'.
 *  @return A Promise for an array of objects containing the metadata of each commit in the range.
 */
// tslint:disable-next-line max-line-length
function gitLog(dir = process.cwd(), startRef, endRef) {
    const rangeString = getRangeString(startRef, endRef);
    return wrapProcess('git', gitLogArgs(rangeString), dir).then(d => parseData(d));
}
exports.gitLog = gitLog;
/**
 *  Synchronously fetches the metadata of all commits within a particular reference range.
 *  @param dir The path to the root directory of a git repository.
 *  @param startRef A reference string, such as a commit hash or tag name, which designates the
 *    beginning of the range (exclusive.) If not defined, all ancestors of endRef will be listed.
 *  @param endRef A reference string, such as a commit hash or tag name, which designates the end
 *    of the range (inclusive.) If not defined, endRef will be assumed to be 'HEAD'.
 *  @return An array of objects containing the metadata of each commit in the range.
 */
// tslint:disable-next-line max-line-length
function gitLogSync(dir = process.cwd(), startRef, endRef) {
    const rangeString = getRangeString(startRef, endRef);
    const child = child_process_1.spawnSync('git', gitLogArgs(rangeString), { cwd: dir });
    if (child.status !== 0) {
        throw getErrorWithCode(child.status, child.stderr.toString());
    }
    return parseData(child.stdout.toString());
}
exports.gitLogSync = gitLogSync;
function getRangeString(startRef, endRef = 'HEAD') {
    if (startRef !== undefined) {
        return `${startRef}..${endRef}`;
    }
    return endRef;
}
function wrapProcess(cmd, args, dir) {
    return new Promise((resolve, reject) => {
        const child = child_process_1.spawn(cmd, args, { cwd: dir });
        let stdout = new Buffer('');
        let stderr = new Buffer('');
        child.stdout.on('data', data => stdout = Buffer.concat([stdout, data]));
        child.stderr.on('data', data => stderr = Buffer.concat([stderr, data]));
        child.once('exit', (code) => {
            if (code !== 0 || stderr.toString() !== '') {
                reject(getErrorWithCode(code, stderr.toString()));
            }
            else {
                resolve(stdout.toString());
            }
        });
    });
}
function parseData(rawData) {
    const commits = [];
    const commitData = rawData.split('\x1E');
    for (let i = 0; i < commitData.length - 1; i += 1) {
        const entries = commitData[i].split('\x1F');
        commits[i] = {
            fullHash: entries[1],
            partialHash: entries[2],
            author: {
                name: entries[3],
                email: entries[4],
            },
            authorTime: new Date(parseInt(entries[5], 10) * 1000),
            committer: {
                name: entries[6],
                email: entries[7],
            },
            commitTime: new Date(parseInt(entries[8], 10) * 1000),
            subject: entries[9],
            body: entries[10],
        };
    }
    return commits;
}
function getErrorWithCode(code, message) {
    const err = new Error(message);
    err.code = code;
    return err;
}
